/**

    Pola:
    
        Node first - atrapa pocztku listy
        
        Node last  - atrapa koca listy
        
        int  size  - aktualny rozmiar listy ( = ilo
                     elementw  n i e  liczc atrap) 
                     
    Konstruktor:
    
        DoubleLinkedList()
            tworzy pust list z dwoma kocami-atrapami
            
    Metody: (przy prbie pobrania elementu z listy pustej 
             wysyany jest wyjtek java.util.NoSuchElement; 
             przy odwoaniu do wza o zbyt maej lub zbyt 
             duej liczbie porzdkowej wysyany jest wyjtek
             IllegalArgumentException)
             
    
        boolean isEmpty() - zwraca true jeli lista jest pusta,
          false w przeciwnym przypadku.
            
        int size() - zwraca aktualny rozmiar listy (czyli 
          ilo elementw  n i e  liczc atrap).
    
        Object retrieve(int p) - zwraca odnonik do elementu
          z p-tego wza listy (liczc od 1).

        void push() - dodaje nowy wze na poczatek

        Object pop() - zwraca element z pierwszego wza listy
          i usuwa ten wze.

        void inject() - dodaje nowy wze na koniec listy.

        void insert(int pos,....) - dodaje nowy 
          wze na pozycji pos (liczc od 1).

        String toString() - wypisuje zawarto listy
/**/

public class DoubleLinkedList
{
    private Node first, last;
    private int  size;

    public DoubleLinkedList()
    {
        first = new Node();
        last  = new Node();
        size  = 0;
        first.setNext(last);
        last.setPrev(first);
    }

    public boolean isEmpty() { return size == 0; }
    public int     size()    { return size;      }
    
    public Node retrieve(int pos)
    {
        if ( (pos < 1) || (pos > size) ) 
            throw new IllegalArgumentException(
                "\nPOS = " + pos + " A SIZE = " + size + " W " + 
                "\"DoubleLinkedList.retrieve()\"");
        Node v = first.getNext();
        for ( int i = 1; i < pos; i++ ) v = v.getNext();
        return v;
    }

    public void push(Node n)
    {
        Node p = first.getNext();
        Node v = new Node(n.getFirstName(), n.getLastName(), n.getPesel(), n.getResults(), p, first);
        first.setNext(v);
        p.setPrev(v);
        size++;
    }

    public Node pop()
    {
        if (isEmpty()) 
            throw new java.util.NoSuchElementException(
                "\nPUSTA LISTA W \"DoubleLinkedList.pop()\"");
        Node p = first.getNext();
        first.setNext(p.getNext());
        p.getNext().setPrev(first);
        size--;
        return p;
    }

    public void inject(String fn, String ln, String pesel, String results)
    {
        Node p = last.getPrev();
        Node v = new Node(fn, ln, pesel, results, last, p);
        p.setNext(v);
        last.setPrev(v);
        size++;
    }
    
    public void inject(Node n)
    {
        Node p = last.getPrev();
        Node v = new Node(n.getFirstName(), n.getLastName(), n.getPesel(), n.getResults(), last, p);
        p.setNext(v);
        last.setPrev(v);
        size++;
    }

    public void insert(int pos, String fn, String ln, String pesel, String results)
    {
        if ( (pos < 1) || (pos > size+1) ) 
            throw new IllegalArgumentException(
                "\nPOS = " + pos + " A SIZE = " + size + " W " +
                "\"DoubleLinkedList.insert()\"");
        Node p = first;
        for ( int i = 1; i < pos; i++ ) p = p.getNext();
        Node v = new Node(fn, ln, pesel, results, p.getNext(), p);
        p.getNext().setPrev(v);
        p.setNext(v);
        size++;
    }

    public String toString() // mozna usunac wypisanie zawartosci listy na konsole
    {
        if ( size == 0 ) return "   LISTA: PUSTA";
        StringBuffer s = new StringBuffer("   LISTA:\n");
        Node p = first;
        for ( int i = 1; i <= size; i++ ) {
            p = p.getNext();
            s.append(p.getFirstName() + " " + p.getLastName() + " " + p.getPesel() + " " +p.getResults() + "\n");
        }
        return s.toString();
    }
    
    public DoubleLinkedList RadixSort(int value)
    {
        DoubleLinkedList tmp = new DoubleLinkedList(); // lista do przechowywania slow o odpowiedniej dlugosci
        DoubleLinkedList tmp1 = new DoubleLinkedList(); // segregowanie tych slow
        DoubleLinkedList tmp2 = new DoubleLinkedList(); // pozostale slowa
        
        int maxLength = 0;
        int size = this.size();
        
        if(value == 0) {
            for (int i = 1; i <= size; i++) {  //szukamy najdluzszego napisu
                if (this.retrieve(i).getFirstName().length() > maxLength)
                    maxLength = this.retrieve(i).getFirstName().length();
            }

            for (int i = maxLength; i > 0; i--)  // zaczynajac od najdluzszego idziemy w dol
            {   
                int p = 0;

                for (int t = 1; t <= size; t++) { //zapisujemy do 'tmp' napisy dluzsze lub rowne niz aktualnie badany 
                    if (this.retrieve(t).getFirstName().length() >= i) {
                        tmp.inject(this.retrieve(t));
                    } else
                        tmp2.inject(this.retrieve(t)); // do 'tmp2' wpisujemy napisy krotsze
                }

                for (char m = 'A'; m <= 'Z' ; m++) {   //bierzemy znak i badamy
                    p = 1;
                    while ( (p <= tmp.size()) ){ //dla wszystkich napisow z listy
                        if((tmp.retrieve(p).getFirstName().charAt(i-1) == m) ||  // czy na danej pozycji jest obecna litera
                           (tmp.retrieve(p).getFirstName().charAt(i-1) == m+32)){ // albo jej maly odpowiednik
                              tmp1.inject(tmp.retrieve(p));  // tu zapisuja sie nam w kolejnosci alfabetycznej(dla odpowiedniego indeksu) napisy
                        }
                        p++;
                    }  
                } 

                for (char m = '0'; m <= '9' ; m++) { // to samo dla liczb ale liczby jak napisy '111' jest wyzej niz '2'
                    p = 1;
                    while ( (p <= tmp.size()) ){
                        if((tmp.retrieve(p).getFirstName().charAt(i-1) == m)){
                            tmp1.inject(tmp.retrieve(p)); 
                        }
                        p++;
                    }  
                }

                while(!tmp2.isEmpty()) // wpisujemy na poczatek listy 'tmp1' to co bylo w 'tmp2'
                    tmp1.push(tmp2.pop());

                while (!this.isEmpty()) //czyscimy aktualna liste(liste glowna)
                    this.pop();

                while (!tmp1.isEmpty())  // czyszczenie listy 'tmp1' i jednoczesine zapis jej elementow do listy glownej
                    this.inject(tmp1.pop());

                while (!tmp.isEmpty()) //czyszczenie listy 'tmp'
                    tmp.pop();   
            } 
        }
        
        if(value == 1) {
            for (int i = 1; i <= size; i++) {  //szukamy najdluzszego napisu
                if (this.retrieve(i).getLastName().length() > maxLength)
                    maxLength = this.retrieve(i).getLastName().length();
            }

            for (int i = maxLength; i > 0; i--)  // zaczynajac od najdluzszego idziemy w dol
            {   
                int p = 0;

                for (int t = 1; t <= size; t++) { //zapisujemy do 'tmp' napisy dluzsze lub rowne niz aktualnie badany 
                    if (this.retrieve(t).getLastName().length() >= i) {
                        tmp.inject(this.retrieve(t));
                    } else
                        tmp2.inject(this.retrieve(t)); // do 'tmp2' wpisujemy napisy krotsze
                }

                for (char m = 'A'; m <= 'Z' ; m++) {   //bierzemy znak i badamy
                    p = 1;
                    while ( (p <= tmp.size()) ){ //dla wszystkich napisow z listy
                        if((tmp.retrieve(p).getLastName().charAt(i-1) == m) ||  // czy na danej pozycji jest obecna litera
                           (tmp.retrieve(p).getLastName().charAt(i-1) == m+32)){ // albo jej maly odpowiednik
                              tmp1.inject(tmp.retrieve(p));  // tu zapisuja sie nam w kolejnosci alfabetycznej(dla odpowiedniego indeksu) napisy
                        }
                        p++;
                    }  
                } 

                for (char m = '0'; m <= '9' ; m++) { // to samo dla liczb ale liczby jak napisy '111' jest wyzej niz '2'
                    p = 1;
                    while ( (p <= tmp.size()) ){
                        if((tmp.retrieve(p).getLastName().charAt(i-1) == m)){
                            tmp1.inject(tmp.retrieve(p)); 
                        }
                        p++;
                    }  
                }

                while(!tmp2.isEmpty()) // wpisujemy na poczatek listy 'tmp1' to co bylo w 'tmp2'
                    tmp1.push(tmp2.pop());

                while (!this.isEmpty()) //czyscimy aktualna liste(liste glowna)
                    this.pop();

                while (!tmp1.isEmpty())  // czyszczenie listy 'tmp1' i jednoczesine zapis jej elementow do listy glownej
                    this.inject(tmp1.pop());

                while (!tmp.isEmpty()) //czyszczenie listy 'tmp'
                    tmp.pop();   
            } 
        }
        
        if(value == 2) {
            for (int i = 1; i <= size; i++) {  //szukamy najdluzszego napisu
                if (this.retrieve(i).getPesel().length() > maxLength)
                    maxLength = this.retrieve(i).getPesel().length();
            }

            for (int i = maxLength; i > 0; i--)  // zaczynajac od najdluzszego idziemy w dol
            {   
                int p = 0;

                for (int t = 1; t <= size; t++) { //zapisujemy do 'tmp' napisy dluzsze lub rowne niz aktualnie badany 
                    if (this.retrieve(t).getPesel().length() >= i) {
                        tmp.inject(this.retrieve(t));
                    } else
                        tmp2.inject(this.retrieve(t)); // do 'tmp2' wpisujemy napisy krotsze
                }

                for (char m = 'A'; m <= 'Z' ; m++) {   //bierzemy znak i badamy
                    p = 1;
                    while ( (p <= tmp.size()) ){ //dla wszystkich napisow z listy
                        if((tmp.retrieve(p).getPesel().charAt(i-1) == m) ||  // czy na danej pozycji jest obecna litera
                           (tmp.retrieve(p).getPesel().charAt(i-1) == m+32)){ // albo jej maly odpowiednik
                              tmp1.inject(tmp.retrieve(p));  // tu zapisuja sie nam w kolejnosci alfabetycznej(dla odpowiedniego indeksu) napisy
                        }
                        p++;
                    }  
                } 

                for (char m = '0'; m <= '9' ; m++) { // to samo dla liczb ale liczby jak napisy '111' jest wyzej niz '2'
                    p = 1;
                    while ( (p <= tmp.size()) ){
                        if((tmp.retrieve(p).getPesel().charAt(i-1) == m)){
                            tmp1.inject(tmp.retrieve(p)); 
                        }
                        p++;
                    }  
                }

                while(!tmp2.isEmpty()) // wpisujemy na poczatek listy 'tmp1' to co bylo w 'tmp2'
                    tmp1.push(tmp2.pop());

                while (!this.isEmpty()) //czyscimy aktualna liste(liste glowna)
                    this.pop();

                while (!tmp1.isEmpty())  // czyszczenie listy 'tmp1' i jednoczesine zapis jej elementow do listy glownej
                    this.inject(tmp1.pop());

                while (!tmp.isEmpty()) //czyszczenie listy 'tmp'
                    tmp.pop();   
            } 
        }
        
        if(value == 3) {
        /* --  kod do sortowania liczb  
           --  pamietac o konwersji Stringa na int   
           --  try{
                 int a = Intrger.praseInt("123");
               }catch(Exception e( {}
        
        */
        }
        
        return this; // zwracamy posortowana liste elementow
    }
}
